@@ -1,6 +1,8 @@
 /* 
 qrq - High speed morse trainer, similar to DL4MM's Rufz    
-Copyright (C) 2006-2007  Fabian Kurz
+Copyright (C) 2006-2008  Fabian Kurz
+
+$Id: qrq.c 41 2008-06-22 19:23:26Z dj1yfk $
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -26,15 +28,14 @@ Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <ctype.h>
 #include <time.h> 
 #include <math.h>
-#include <sys/ioctl.h>
 #include <fcntl.h>
-#include <sys/soundcard.h>
 #include <unistd.h>
 #include <sys/stat.h>			/* mkdir */
 #include <sys/types.h>
+#include <errno.h>
 
 
-#define PI 3.1415926
+#define PI M_PI
 
 #define SILENCE 0		/* Waveforms for the tone generator */
 #define SINE 1
@@ -49,10 +50,18 @@ Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #  define VERSION "0.0.0"
 #endif
 
+#ifndef OPENAL
+#include "oss.h"
+#define write_audio(x, y, z) write(x, y, z)
+#define close_audio(x) close(x)
+typedef int AUDIO_HANDLE;
+#else
+#include "OpenAlImp.h"
+typedef void *AUDIO_HANDLE;
+#endif
 
-
-
-static char calls[20084][15];
+/* callsign array will be dynamically allocated */
+static char **calls;
 const static char *codetable[] = {
 ".-", "-...", "-.-.", "-..", ".", "..-.", "--.", "....", "..",".---",
 "-.-",".-..","--","-.","---",".--.","--.-",".-.","...","-","..-","...-",
@@ -60,10 +69,10 @@ const static char *codetable[] = {
 "-....", "--...","---..","----."};
 
 static char mycall[15]="DJ1YFK";		/* mycall. will be read from qrqrc */
-static char dspdevice[15]="/dev/dsp";	/* will also be read from qrqrc */
+static char dspdevice[80]="/dev/dsp";	/* will also be read from qrqrc */
 static int score = 0;					/* qrq score */
 static int initialspeed=200;			/* initial speed. to be read from file*/
-static int speed=200;					/* current speed in wpm */
+static int speed=200;					/* current speed in cpm */
 static int maxspeed=0;
 static int freq=800;					/* current cw sidetone freq */
 static int errornr=0;					/* number of errors in attempt */
@@ -75,7 +84,7 @@ static int constanttone=0;              /* if 1 don't change the pitch */
 static int ctonefreq=800;               /* if constanttone=1 use this freq */
 
 
-static long samplerate=44100;
+long samplerate=44100;
 static long long_i;
 static int waveform = SINE;				/* waveform: (0 = none) */
 static char wavename[10]="Sine    ";	/* Name of the waveform */
@@ -86,7 +95,7 @@ static int ft;							/* falltime. to be calced in 'morse' */
 
 static short buffer[88200];
 
-static int dsp_fd;
+static AUDIO_HANDLE dsp_fd;
 
 static int display_toplist(WINDOW * win);
 static int calc_score (char * realcall, char * input, int speed, char * output);
@@ -98,23 +107,34 @@ static int read_config();
 static int save_config();
 static int tonegen(int freq, int length, int waveform);
 static void *morse(void * arg); 
-static int open_dsp (char * device); 
 static int readcall(WINDOW *win, int y, int x, char * call); 
 static void thread_fail (int j);
 static int check_toplist ();
 static int find_files ();
 static int statistics ();
+static int read_callbase ();
 
 pthread_t cwthread;				/* thread for CW output, to enable
 								   keyboard reading at the same time */
 pthread_attr_t cwattr;
 
-char rcfilename[80]="";			/* filename and path to qrqrc */
-char tlfilename[80]="";			/* filename and path to toplist */
-char cbfilename[80]="";			/* filename and path to callbase */
+char rcfilename[1024]="";			/* filename and path to qrqrc */
+char tlfilename[1024]="";			/* filename and path to toplist */
+char cbfilename[1024]="";			/* filename and path to callbase */
 
+char destdir[1024]="";
 
 int main (int argc, char *argv[]) {
+
+  /* if built as osx bundle set DESTDIR to Resources dir of bundle */
+#ifdef OSX_BUNDLE
+  char* p_slash = strrchr(argv[0], '/');
+  strncpy(destdir, argv[0], p_slash - argv[0]);
+  strcat(destdir, "/../Resources");
+#else
+  strcpy(destdir, DESTDIR);
+#endif
+
 	char tmp[80]="";
 	char input[15]="";
 	int i=0;						/* counter etc. */
@@ -128,7 +148,7 @@ int main (int argc, char *argv[]) {
 	WINDOW *right_w;				/* highscore list/settings		*/
 
 	if (argc > 1) {
-		printf("qrq v%s  (c) 2006-2007 Fabian Kurz, DJ1YFK. "
+		printf("qrq v%s  (c) 2006-2008 Fabian Kurz, DJ1YFK. "
 					"http://fkurz.net/ham/qrq.html\n", VERSION);
 		printf("High speed morse telegraphy trainer, similar to"
 					" DL4MM's RUFZ.\n\n");
@@ -147,7 +167,7 @@ int main (int argc, char *argv[]) {
 	keypad(stdscr, TRUE);
 	scrollok(stdscr, FALSE);
 
-	printw("qrq v%s - Copyright (C) 2006-2007 Fabian Kurz, DJ1YFK\n", VERSION);
+	printw("qrq v%s - Copyright (C) 2006-2008 Fabian Kurz, DJ1YFK\n", VERSION);
 	printw("This is free software, and you are welcome to redistribute it\n");
 	printw("under certain conditions (see COPYING).\n");
 
@@ -180,22 +200,10 @@ int main (int argc, char *argv[]) {
 	/****** Reading configuration file ******/
 	printw("\nReading configuration file qrqrc \n");
 	read_config();
-	
+
 	/****** Reading callsign database ******/
-	printw("\n");
-	printw("Reading callsign database... ");
-	if ((fh = fopen(cbfilename, "r")) == NULL) {
-		endwin();
-		fprintf(stderr, "Error: Couldn't read callsign database ('%s')!\n",
-						cbfilename);
-		exit(EXIT_FAILURE);
-	}
-	while (fgets(tmp,15,fh) != NULL) {
-		tmp[strlen(tmp)-1]='\0';				/* remove newline */
-		strcpy(calls[nrofcalls],tmp);
-		nrofcalls++;
-	}
-	fclose(fh);
+	printw("\nReading callsign database... ");
+	nrofcalls = read_callbase();
 
 	printw("done. %d calls read.\n\n", nrofcalls);
 	printw("Press any key to continue...");
@@ -204,14 +212,9 @@ int main (int argc, char *argv[]) {
 	getch();
 
 	top_w = newwin(4, 60, 0, 0);
-	keypad(top_w, TRUE);
 	mid_w = newwin(17, 60, 4, 0);
-	keypad(mid_w, TRUE);
 	bot_w = newwin(3, 60, 21, 0);
-	keypad(bot_w, TRUE);
 	right_w = newwin(24, 20, 0, 60);
-	keypad(right_w, TRUE);
-
 	
 	/* no need to join here, this is the first possible time CW is sent */
 	pthread_create(&cwthread, NULL, & morse, (void *) "QRQ");		
@@ -269,16 +272,17 @@ while (status == 1) {
 	maxspeed = errornr = score = 0;
 	speed = initialspeed;
 	
-	/* promt for own callsign */
+	/* prompt for own callsign */
 	i = readcall(bot_w, 1, 30, mycall);
 
 	/* F5 -> Configure sound */
 	if (i == 5) {
 		status = 2;
-		break;			/* get out of the while loop */
+		break;
 	} 
 	/* F6 -> play test CW */
 	else if (i == 6) {
+		freq = constanttone ? ctonefreq : 800;
 		pthread_join(cwthread, NULL);
 		j = pthread_create(&cwthread, NULL, &morse, (void *) "VVVTEST");	
 		thread_fail(j);
@@ -328,7 +332,7 @@ while (status == 1) {
 		}
 		else {
 				/* fixed frequency */
-				freq=ctonefreq ;
+				freq = ctonefreq;
 		}
 
 		mvwprintw(bot_w,1,1,"                                      ");
@@ -494,6 +498,7 @@ while (status == 2) {
 			sleep(1);	
 			break;
 		case KEY_F(6):
+			freq = constanttone ? ctonefreq : 800;
 			pthread_join(cwthread, NULL);
 			j = pthread_create(&cwthread, NULL, &morse, (void *) "TESTING");	
 			thread_fail(j);
@@ -524,8 +529,6 @@ while (status == 2) {
 static int readcall(WINDOW *win, int y, int x, char * call) {
 	int c;						/* character we read */
 	int i=0;
-	char tmp[2]=" ";			/* temp string to concatenate to call */
-	tmp[1]='\0';
 
 	if (strlen(call) == 0) {p=0;}	/* cursor to start if no call in buffer */
 	
@@ -636,10 +639,10 @@ static int display_toplist (WINDOW * win) {
 		exit(EXIT_FAILURE);
 	}
 	rewind(fh);				/* a+ -> end of file, we want the beginning */
-	(void) fgets(tmp, 35, fh);		/* first line is crap */
+	(void) fgets(tmp, 34, fh);		/* first line not used */
 	while ((feof(fh) == 0) && i < 20) {
 		i++;
-		if (fgets(tmp, 35, fh) != NULL) {
+		if (fgets(tmp, 34, fh) != NULL) {
 			tmp[17]='\0';
 			if (strstr(tmp, mycall)) {		/* highlight own call */
 				wattron(win, A_BOLD);
@@ -775,7 +778,7 @@ static int add_to_toplist(char * mycall, int score, int maxspeed) {
 		tmp[i] = '\0';
 		i = atoi(tmp);				/* i = score of current line */
 		if (i < score){ 			/* insert score and shift lines below */
-			score = i;				/* ... looks ugly, and it's buggy */
+			score = i;				/* (ugly) */
 			fseek(fh, -32L, SEEK_CUR);
 			fputs(insertline, fh);
 			strcpy(insertline, line);	/* actual line -> print one later */
@@ -929,6 +932,9 @@ static void *morse(void *arg) {
 	/* opening the DSP device */
 	dsp_fd = open_dsp(dspdevice);
 
+	/* Some silence; otherwise the call starts right after pressing enter */
+	tonegen(0, 250, SILENCE);
+
 	/* speed is in LpM now, so we have to calculate the dot-length in
 	 * milliseconds using the well-known formula  dotlength= 60/(wpm*50) */
 
@@ -972,8 +978,8 @@ static void *morse(void *arg) {
 		tonegen(0, 2*ms, SILENCE);
 	}
 
-	write(dsp_fd, buffer, 88200);
-	close(dsp_fd);
+	write_audio(dsp_fd, buffer, 88200);
+	close_audio(dsp_fd);
 
 	return NULL;
 }
@@ -1012,54 +1018,11 @@ static int tonegen (int freq, int len, int waveform) {
 		
 		out = (int) (val * 32500.0);
 		out = out + (out<<16);				/* add second channel */
-		write(dsp_fd, &out, sizeof(out));
+		write_audio(dsp_fd, &out, sizeof(out));
 	}
 	return 0;
 }
 
-static int open_dsp (char * device) {
-	int tmp;
-	int fd;
-	
-	if ((fd = open(device, O_WRONLY, 0)) == -1) {
-		endwin();
-		perror(device);
-		exit(EXIT_FAILURE);
-	}
-
-	tmp = AFMT_S16_NE; 
-	if (ioctl(fd, SNDCTL_DSP_SETFMT, &tmp)==-1) {
-		endwin();
-		perror("SNDCTL_DSP_SETFMT");
-		exit(EXIT_FAILURE);
-	}
-
-	if (tmp != AFMT_S16_NE) {
-		endwin();
-		fprintf(stderr, "Cannot switch to AFMT_S16_NE\n");
-		exit(EXIT_FAILURE);
-	}
-  
-	tmp = 2;	/* 2 channels, stereo */
-	if (ioctl(fd, SNDCTL_DSP_CHANNELS, &tmp)==-1) {
-		endwin();
-		perror("SNDCTL_DSP_CHANNELS");
-		exit(EXIT_FAILURE);
-	}
-
-	if (tmp != 2) {
-		endwin();
-		fprintf(stderr, "No stereo mode possible :(.\n");
-		exit(EXIT_FAILURE);
-	}
-
-	if (ioctl(fd, SNDCTL_DSP_SPEED, &samplerate)==-1) {
-		endwin();
-		perror("SNDCTL_DSP_SPEED");
-		exit(EXIT_FAILURE);
-	}
-return fd;
-}
 
 static int save_config () {
 	FILE *fh;
@@ -1122,7 +1085,7 @@ static void thread_fail (int j) {
 	}
 }
 
-
+/* Add timestamps to toplist file if not there yet */
 static int check_toplist () {
 	char line[80]="";
 	char tmp[80]="";
@@ -1185,6 +1148,9 @@ static int find_files () {
 
 	FILE *fh;
 	const char *homedir = NULL;
+	char tmp_rcfilename[1024] = "";
+	char tmp_tlfilename[1024] = "";
+	char tmp_cbfilename[1024] = "";
 
 	printw("\nChecking for neccesary files (qrqrc, toplist, callbase)...\n");
 	
@@ -1205,13 +1171,21 @@ static int find_files () {
 		 * DESTDIR/share/qrq/. */
 
 		if ((fh = fopen(rcfilename, "r")) == NULL ) {
-			printw("... not found in %s/.qrq/. Checking "DESTDIR"/share/qrq..."
-							"\n", homedir);
+			printw("... not found in %s/.qrq/. Checking %s/share/qrq..."
+							"\n", homedir, destdir);
 			/* check for the files in DESTDIR/share/qrq/. if exists, copy 
 			 * qrqrc and toplist to ~/.qrq/  */
-			if (((fh = fopen(DESTDIR"/share/qrq/qrqrc", "r")) == NULL) ||
-				((fh = fopen(DESTDIR"/share/qrq/toplist", "r")) == NULL) ||
-				 ((fh = fopen(DESTDIR"/share/qrq/callbase", "r")) == NULL)) {
+
+			strcpy(tmp_rcfilename, destdir);
+			strcat(tmp_rcfilename, "/share/qrq/qrqrc");
+			strcpy(tmp_tlfilename, destdir);
+			strcat(tmp_tlfilename, "/share/qrq/toplist");
+			strcpy(tmp_cbfilename, destdir);
+			strcat(tmp_cbfilename, "/share/qrq/callbase");
+
+			if (((fh = fopen(tmp_rcfilename, "r")) == NULL) ||
+				((fh = fopen(tmp_tlfilename, "r")) == NULL) ||
+				 ((fh = fopen(tmp_cbfilename, "r")) == NULL)) {
 				printw("Sorry: Couldn't find 'qrqrc', 'toplist' and"
 			   			" 'callbase' anywhere. Exit.\n");
 				getch();
@@ -1220,13 +1194,13 @@ static int find_files () {
 			}
 			else {			/* finally found it in DESTDIR/share/qrq/ ! */
 				/* abusing rcfilename here for something else temporarily */
-				printw("Found files in "DESTDIR"/share/qrq/."
+				printw("Found files in %s/share/qrq/."
 						"\nCreating directory %s/.qrq/ and copy qrqrc and"
-						" toplist there.\n", homedir);
+						" toplist there.\n", destdir, homedir);
 				strcpy(rcfilename, homedir);
 				strcat(rcfilename, "/.qrq/");
 				j = mkdir(rcfilename,  0777);
-				if (j) {
+				if (j && (errno != EEXIST)) {
 					printw("Failed to create %s! Exit.\n", rcfilename);
 					getch();
 					endwin();
@@ -1236,7 +1210,9 @@ static int find_files () {
 				 * DESTDIR/local/, so I assume copying files won't cause any
 				 * problem, with system()... */
 
-				strcpy(rcfilename, "install -m 644 "DESTDIR"/share/qrq/toplist "); 
+				strcpy(rcfilename, "install -m 644 ");
+				strcat(rcfilename, tmp_tlfilename);
+				strcat(rcfilename, " ");
 				strcat(rcfilename, homedir);
 				strcat(rcfilename, "/.qrq/ 2> /dev/null");
 				if (system(rcfilename)) {
@@ -1245,7 +1221,9 @@ static int find_files () {
 					endwin();
 					exit(EXIT_FAILURE);
 				}
-				strcpy(rcfilename, "install -m 644 "DESTDIR"/share/qrq/qrqrc "); 
+				strcpy(rcfilename, "install -m 644 ");
+				strcat(rcfilename, tmp_rcfilename);
+				strcat(rcfilename, " ");
 				strcat(rcfilename, homedir);
 				strcat(rcfilename, "/.qrq/ 2> /dev/null");
 				if (system(rcfilename)) {
@@ -1260,14 +1238,15 @@ static int find_files () {
 				strcat(rcfilename, "/.qrq/qrqrc");
 				strcpy(tlfilename, homedir);
 				strcat(tlfilename, "/.qrq/toplist");
-				strcpy(cbfilename, DESTDIR"/share/qrq/callbase");
+				strcpy(cbfilename, tmp_cbfilename);
 			} /* found in DESTDIR/share/qrq/ */
 		}
 		else {
 			printw("... found files in %s/.qrq/.\n", homedir);
 			strcat(tlfilename, homedir);
 			strcat(tlfilename, "/.qrq/toplist");
-			strcpy(cbfilename, DESTDIR"/share/qrq/callbase");
+			strcpy(cbfilename, destdir);
+			strcat(cbfilename, "/share/qrq/callbase");
 		}
 	}
 	else {
@@ -1330,3 +1309,58 @@ static int statistics () {
 		system("gnuplot /tmp/qrq-plot 2> /dev/null &");
 	return 0;
 }
+
+
+int read_callbase () {
+	FILE *fh;
+	int c;
+	char tmp[80] = "";
+	int nr=0;
+
+	if ((fh = fopen(cbfilename, "r")) == NULL) {
+		endwin();
+		fprintf(stderr, "Error: Couldn't read callsign database ('%s')!\n",
+						cbfilename);
+		exit(EXIT_FAILURE);
+	}
+
+	/* count the lines/calls */
+	while ((c = getc(fh)) != EOF) {
+		if (c == '\n') {
+			nr++;
+		}
+	}
+
+	/* allocate memory for calls array */
+
+	if ((calls = (char **) malloc( (size_t) sizeof(char *)*nr )) == NULL) {
+		fprintf(stderr, "Error: Couldn't allocate %d bytes!\n", 
+						(int) sizeof(char)*nr*15);
+		exit(EXIT_FAILURE);
+	}
+	
+	/* Allocate each element of the array with size 15 */
+	for (c=0; c < nr; c++) {
+		if ((calls[c] = (char *) malloc (15 * sizeof(char))) == NULL) {
+			fprintf(stderr, "Error: Couldn't allocate 15 bytes!\n");
+			exit(EXIT_FAILURE);
+		}
+	}
+
+	rewind(fh);
+	
+	nr=0;
+	while (fgets(tmp,15,fh) != NULL) {
+		tmp[strlen(tmp)-1]='\0';				/* remove newline */
+		strcpy(calls[nr],tmp);
+		nr++;
+		if (nr == c) 			/* may happen if call file corrupted */
+				break;
+	}
+	fclose(fh);
+
+	return nr;
+
+}
+
+
